/**
 * @file
 *
 * Database types, data generation and loading functionality for memory mapped files (header).
 *
 * @author Henning Funke <henning.funke@cs.tu-dortmund.de>
 *
 * (c) 2019 TU Dortmund, DBIS Group
 */
#pragma once

#include <cstddef>

typedef long int Tuple;

typedef struct Relation {
    Tuple* r;
    size_t len;
    size_t capacity;
} Relation;


/**
  * @brief Allocate tuple array and initialize Relation attributes.
  */
Relation allocateRelation ( size_t capacity );


/**
  * @brief Free tuple array.
  */
void freeRelation ( Relation col );


/**
  * @brief Generate relation with uniform distribution in memory mapped file.
  * Fills the fields of out with corresponding sizes and pointers.
  * The generated file may be deleted to overwrite/free.
  */
void genData ( Relation* out, const char* filepath, size_t len );


/**
  * @brief Load relation from memory mapped fill into out.
  * Returns whether loading and size verification was successful.
  */
bool loadData ( Relation* out, const char* filepath, size_t len );


